@@ -41,6 +41,7 @@ class ClipBoard;
 class KURL;
 class MultiTrackReader;
 class MultiTrackWriter;
+class SignalWidget;
 class Track;
 class UndoAction;
 class UndoTransaction;
@@ -288,6 +289,16 @@ public:
     /** Returns true if undo/redo is currently enabled */
     inline bool undoEnabled() { return m_undo_enabled; };
 
+    /** Return true if undo is possible */
+    inline bool canUndo() {
+	return !m_undo_buffer.isEmpty() && undoEnabled();
+    };
+
+    /** Return true if redo is possible */
+    inline bool canRedo() {
+	return !m_redo_buffer.isEmpty() && undoEnabled();
+    };
+
     /**
      * Enables undo and redo. If undo/redo is already enabled, nothing
      * will be done.
@@ -501,6 +512,7 @@ protected:
 
     friend class UndoTransactionGuard;
     friend class PluginManager;
+    friend class SignalWidget;
 
     /**
      * Tries to free memory for a new undo action and stores all needed
