@@ -20,8 +20,12 @@
 #include <math.h>
 #include <stdlib.h>
 
+#include <qbitmap.h>
 #include <qdragobject.h>
+#include <qevent.h>
+#include <qpopupmenu.h>
 #include <qstrlist.h>
+#include <qtooltip.h>
 
 #include <kcursor.h>
 #include <kglobal.h>
@@ -29,6 +33,7 @@
 #include <kmessagebox.h>
 #include <klocale.h>
 #include <kfiledialog.h>
+#include <kiconloader.h>
 
 #include "libkwave/FileInfo.h"
 #include "libkwave/KwaveDrag.h"
@@ -42,6 +47,7 @@
 #include "libkwave/Signal.h"
 #include "libkwave/Track.h"
 
+#include "libgui/LabelPropertiesWidget.h"
 #include "libgui/MenuManager.h"
 #include "libgui/TrackPixmap.h"
 
@@ -50,6 +56,9 @@
 #include "SignalManager.h"
 #include "MouseMark.h"
 #include "UndoTransactionGuard.h"
+#include "UndoAddLabelAction.h"
+#include "UndoDeleteLabelAction.h"
+#include "UndoModifyLabelAction.h"
 
 #ifdef DEBUG
 #include <time.h>
@@ -80,6 +89,16 @@
 /** number of milliseconds between repaints */
 #define REPAINT_INTERVAL 333
 
+/** number of milliseconds until the position widget disappears */
+#define POSITION_WIDGET_TIME 5000
+
+/**
+ * tolerance in pixel for snapping to a label or selection border
+ * @todo selection tolerance should depend on some KDE setting,
+ *        but which ?
+ */
+#define SELECTION_TOLERANCE 10
+
 //***************************************************************************
 //***************************************************************************
 class KwaveFileDrag: public QUriDrag
@@ -108,9 +127,10 @@ SignalWidget::SignalWidget(QWidget *parent)
     m_zoom(0.0), m_playpointer(-1), m_last_playpointer(-1), m_redraw(false),
     m_inhibit_repaint(0), m_selection(0),
     m_signal_manager(this),
-    m_track_pixmaps(), m_pixmap(0),
+    m_track_pixmaps(), m_labels(), m_pixmap(0),
     m_mouse_mode(MouseNormal),
-    m_mouse_down_x(0), m_repaint_timer()
+    m_mouse_down_x(0), m_repaint_timer(this),
+    m_position_widget(this), m_position_widget_timer(this)
 {
 //    qDebug("SignalWidget::SignalWidget()");
 
@@ -156,10 +176,9 @@ SignalWidget::SignalWidget(QWidget *parent)
     connect(&m_repaint_timer, SIGNAL(timeout()),
             this, SLOT(timedRepaint()));
 
-//    labels = new LabelList();
-//    Q_ASSERT(labels);
-//    if (!labels) return;
-//    labels->setAutoDelete (true);
+    // connect the timer of the position widget
+    connect(&m_position_widget_timer, SIGNAL(timeout()),
+            &m_position_widget, SLOT(hide()));
 
 //    m_menu_manager.clearNumberedMenu("ID_LABELS_TYPE");
 //    for (LabelType *tmp = globals.markertypes.first(); tmp;
@@ -192,12 +211,17 @@ void SignalWidget::refreshSignalLayer()
     refreshLayer(LAYER_SIGNAL);
 }
 
+//***************************************************************************
+void SignalWidget::refreshMarkersLayer()
+{
+    refreshLayer(LAYER_MARKERS);
+}
+
 //***************************************************************************
 bool SignalWidget::isOK()
 {
-////    Q_ASSERT(labels);
     Q_ASSERT(m_selection);
-    return ( /* labels && */ m_selection );
+    return ( m_selection );
 }
 
 //***************************************************************************
@@ -208,8 +232,7 @@ SignalWidget::~SignalWidget()
     if (m_pixmap) delete m_pixmap;
     m_pixmap = 0;
 
-////    if (labels) delete labels;
-////    labels = 0;
+    m_labels.clear();
 
     if (m_selection) delete m_selection;
     m_selection = 0;
@@ -291,15 +314,13 @@ bool SignalWidget::executeNavigationCommand(const QString &command)
 	selectRange(m_offset, pixels2samples(m_width) - 1);
     CASE_COMMAND("selectnone")
 	selectRange(m_offset, 0);
-    } else return false;
-
-    return true;
-}
-
-//***************************************************************************
-//bool SignalWidget::executeLabelCommand(const QString &command)
-//{
-//    if (false) {
+    // label commands
+    CASE_COMMAND("label")
+	unsigned int pos = parser.toUInt();
+	addLabel(pos);
+    CASE_COMMAND("deletelabel")
+	int index = parser.toInt();
+	deleteLabel(index, true);
 //    CASE_COMMAND("chooselabel")
 //	Parser parser(command);
 //	markertype = globals.markertypes.at(parser.toInt());
@@ -307,32 +328,22 @@ bool SignalWidget::executeNavigationCommand(const QString &command)
 //	markSignal(command);
 //    CASE_COMMAND("pitch")
 //	markPeriods(command);
-////    CASE_COMMAND("labeltopitch")
-////      convertMarkstoPitch(command);
-//    CASE_COMMAND("deletelabel")
-//	deleteLabel();
-//    CASE_COMMAND("insertlabel")
-//	appendLabel();
+//    CASE_COMMAND("labeltopitch")
+//      convertMarkstoPitch(command);
 //    CASE_COMMAND("loadlabel")
 //	loadLabel();
 //    CASE_COMMAND("savelabel")
 //	saveLabel(command);
-//    CASE_COMMAND("label")
-//	addLabel(command);
-//    CASE_COMMAND("newlabeltype")
-//	addLabelType(command);
 //    CASE_COMMAND("expandtolabel")
 //	jumptoLabel();
-//    CASE_COMMAND("mark")
-//	markSignal(command);
 //    CASE_COMMAND("markperiod")
 //	markPeriods(command);
 //    CASE_COMMAND("saveperiods")
 //	savePeriods();
-//    } else return false;
-//
-//    return true;
-//}
+    } else return false;
+
+    return true;
+}
 
 //***************************************************************************
 bool SignalWidget::executeCommand(const QString &command)
@@ -354,15 +365,6 @@ bool SignalWidget::executeCommand(const QString &command)
 void SignalWidget::selectRange(unsigned int offset, unsigned int length)
 {
     m_signal_manager.selectRange(offset, length);
-
-    // if the selection has been changed through a command we also
-    // have to update the mouse selection
-    if ( (m_selection->left()  != m_signal_manager.selection().first()) ||
-         (m_selection->right() != m_signal_manager.selection().last()))
-    {
-	m_selection->set(m_signal_manager.selection().first(),
-	                 m_signal_manager.selection().last());
-    }
 }
 
 //***************************************************************************
@@ -460,6 +462,9 @@ void SignalWidget::close()
     m_signal_manager.playbackController().playbackStop();
     m_signal_manager.playbackController().reset();
 
+    // remove all layers
+    m_labels.clear();
+
     // clear the display
     m_track_pixmaps.setAutoDelete(true);
     m_track_pixmaps.clear();
@@ -772,12 +777,7 @@ void SignalWidget::refreshLayer(int layer)
 //***************************************************************************
 int SignalWidget::selectionPosition(const int x)
 {
-    Q_ASSERT(m_selection);
-    if (!m_selection) return None;
-
-    /** @todo selection tolerance should depend on some KDE setting,
-              but which ? */
-    const int tol = 10;
+    const int tol = SELECTION_TOLERANCE;
     const unsigned int first = m_signal_manager.selection().first();
     const unsigned int last  = m_signal_manager.selection().last();
     const unsigned int ofs   = m_offset;
@@ -791,17 +791,20 @@ int SignalWidget::selectionPosition(const int x)
     const int rl = (r > tol) ? (r-tol) : 0;
     const int rr = r + tol;
 
-    // position is in the selection while left and right tolerance
-    // areas overlap or leave an area that is smaller than the
-    // tolerance ?
-    if ((rl-lr < tol) && (x >= l) && (x <= r)) return Selection;
-
     // the simple cases...
     int pos = None;
-    if ((x >= ll) && (x <= lr)) pos = LeftBorder;
-    if ((x >= rl) && (x <= rr)) pos = RightBorder;
+    if ((x >= ll) && (x <= lr) && (x <= r)) pos |= LeftBorder;
+    if ((x >= rl) && (x <= rr) && (x >= l)) pos |= RightBorder;
     if ((x >   l) && (x <   r)) pos |= Selection;
 
+    if ((pos & LeftBorder) && (pos & RightBorder)) {
+	// special case: determine which border is nearer
+	if ((x - l) < (r - x))
+	    pos &= ~RightBorder; // more on the left
+	else
+	    pos &= ~LeftBorder;  // more on the right
+    }
+
     return pos;
 }
 
@@ -811,7 +814,7 @@ bool SignalWidget::isSelectionBorder(int x)
     SelectionPos pos = static_cast<SignalWidget::SelectionPos>(
 	selectionPosition(x) & ~Selection);
 
-    return ((pos == LeftBorder) || (pos == RightBorder));
+    return ((pos & LeftBorder) || (pos & RightBorder));
 }
 
 //***************************************************************************
@@ -839,12 +842,14 @@ void SignalWidget::mousePressEvent(QMouseEvent *e)
 	if (mx < 0) mx = 0;
 	if (mx >= m_width) mx = m_width-1;
 	unsigned int x = m_offset + pixels2samples(mx);
-	unsigned int len = m_selection->right() - m_selection->left() + 1;
+	unsigned int len = m_signal_manager.selection().length();
 	switch (e->state() & KeyButtonMask) {
 	    case ShiftButton: {
 		// expand the selection to "here"
-		selectRange(m_selection->left(), len);
+		m_selection->set(m_signal_manager.selection().first(),
+		                 m_signal_manager.selection().last());
 		m_selection->grep(x);
+		selectRange(m_selection->left(), m_selection->length());
 		setMouseMode(MouseSelect);
 		break;
 	    }
@@ -858,7 +863,10 @@ void SignalWidget::mousePressEvent(QMouseEvent *e)
 	    case 0: {
 		if (isSelectionBorder(e->pos().x())) {
 		    // modify selection border
+		    m_selection->set(m_signal_manager.selection().first(),
+			             m_signal_manager.selection().last());
 		    m_selection->grep(x);
+		    selectRange(m_selection->left(), m_selection->length());
 		    setMouseMode(MouseSelect);
 		} else if (isInSelection(e->pos().x()) && (len > 1)) {
 		    // store the x position for later drag&drop
@@ -866,6 +874,7 @@ void SignalWidget::mousePressEvent(QMouseEvent *e)
 		} else {
 		    // start a new selection
 		    m_selection->set(x, x);
+		    selectRange(x, 0);
 		    setMouseMode(MouseSelect);
 		}
 		break;
@@ -874,6 +883,133 @@ void SignalWidget::mousePressEvent(QMouseEvent *e)
     }
 }
 
+//***************************************************************************
+void SignalWidget::contextMenuEvent(QContextMenuEvent *e)
+{
+    Q_ASSERT(e);
+    Q_ASSERT(m_selection);
+    if (!e || !m_selection) return;
+
+    QPopupMenu *context_menu = new QPopupMenu(this);
+    Q_ASSERT(context_menu);
+    if (!context_menu) return;
+
+    QPopupMenu *submenu_label = new QPopupMenu(this);
+    Q_ASSERT(submenu_label);
+    if (!submenu_label) return;
+
+    SignalManager *manager = &signalManager();
+    KIconLoader icon_loader;
+
+    /* menu items common to all cases */
+
+    // undo/redo
+    int id;
+    id = context_menu->insertItem(
+	icon_loader.loadIcon("undo.png", KIcon::Toolbar),
+	i18n("&Undo"), this, SLOT(contextMenuEditUndo()), CTRL+Key_Z);
+    if (!manager->canUndo())
+	context_menu->setItemEnabled(id, false);
+
+    id = context_menu->insertItem(
+	icon_loader.loadIcon("redo.png", KIcon::Toolbar),
+	i18n("&Redo"), this, SLOT(contextMenuEditRedo()), CTRL+Key_Y);
+    if (!manager->canRedo())
+	context_menu->setItemEnabled(id, false);
+    context_menu->insertSeparator();
+
+    // cut/copy/paste
+    id = context_menu->insertItem(
+	icon_loader.loadIcon("editcut.png", KIcon::Toolbar),
+	i18n("Cu&t"), this, SLOT(contextMenuEditCut()), CTRL+Key_X);
+    id = context_menu->insertItem(
+	icon_loader.loadIcon("editcopy.png", KIcon::Toolbar),
+	i18n("&Copy"), this, SLOT(contextMenuEditCopy()), CTRL+Key_C);
+    id = context_menu->insertItem(
+	icon_loader.loadIcon("editpaste.png", KIcon::Toolbar),
+	i18n("&Paste"), this, SLOT(contextMenuEditPaste()), CTRL+Key_V);
+    context_menu->insertSeparator();
+
+    int mouse_x = mapFromGlobal(e->globalPos()).x();
+    if (mouse_x < 0) mouse_x = 0;
+    if (mouse_x >= width())  mouse_x = width()  - 1;
+    unsigned int len = manager->selection().length();
+
+    // label handling
+    int id_label_new = submenu_label->insertItem(
+	i18n("&New"), this, SLOT(contextMenuLabelNew()));
+    int id_label_delete = submenu_label->insertItem(
+	i18n("&Delete"), this, SLOT(contextMenuLabelDelete()));
+    submenu_label->setItemEnabled(id_label_delete, false);
+    int id_label_properties = submenu_label->insertItem(
+	i18n("&Properties..."), this, SLOT(contextMenuLabelProperties()));
+    submenu_label->setItemEnabled(id_label_properties, false);
+
+    // store the menu position in the mouse selection
+    unsigned int pos = m_offset + pixels2samples(mouse_x);
+    m_selection->set(pos, pos);
+
+    Label *label = 0;
+    if ((label = findLabelNearMouse(mouse_x))) {
+	// delete label ?
+	// label properties ?
+	submenu_label->setItemEnabled(id_label_new, false);
+	submenu_label->setItemEnabled(id_label_properties, true);
+	submenu_label->setItemEnabled(id_label_delete, true);
+
+	pos = label->pos();
+	m_selection->set(pos, pos);
+    } else if (isSelectionBorder(mouse_x)) {
+	// context menu: do something with the selection border
+
+	// expand to next marker (right) ?
+	// expand to next marker (left) ?
+    } else if (isInSelection(mouse_x) && (len > 1)) {
+	// context menu: do something with the selection
+
+	id = context_menu->insertItem(
+	    i18n("&Save selection..."), this,
+	    SLOT(contextMenuSaveSelection()));
+
+	// convert selection to markers ?
+    }
+
+    context_menu->insertItem("&Label", submenu_label);
+
+    context_menu->exec(QCursor::pos());
+    delete context_menu;
+}
+
+//***************************************************************************
+void SignalWidget::contextMenuLabelNew()
+{
+    Q_ASSERT(m_selection);
+    if (!m_selection) return;
+
+    forwardCommand(QString("label(%1)").arg(m_selection->left()));
+}
+
+//***************************************************************************
+void SignalWidget::contextMenuLabelDelete()
+{
+    Q_ASSERT(m_selection);
+    if (!m_selection) return;
+
+    Label *label = findLabel(m_selection->left());
+    int index = labelIndex(label);
+    forwardCommand(QString("deletelabel(%1)").arg(index));
+}
+
+//***************************************************************************
+void SignalWidget::contextMenuLabelProperties()
+{
+    Q_ASSERT(m_selection);
+    if (!m_selection) return;
+
+    Label *label = findLabel(m_selection->left());
+    labelProperties(label);
+}
+
 //***************************************************************************
 void SignalWidget::mouseReleaseEvent(QMouseEvent *e)
 {
@@ -892,11 +1028,9 @@ void SignalWidget::mouseReleaseEvent(QMouseEvent *e)
 	case MouseSelect: {
 	    unsigned int x = m_offset + pixels2samples(e->pos().x());
 	    m_selection->update(x);
-
-	    unsigned int len = m_selection->right() - m_selection->left() + 1;
-	    selectRange(m_selection->left(), len);
-
+	    selectRange(m_selection->left(), m_selection->length());
 	    setMouseMode(MouseNormal);
+	    showPosition(0, 0, 0, QPoint(-1,-1));
 	    break;
 	}
 	case MouseInSelection: {
@@ -906,9 +1040,10 @@ void SignalWidget::mouseReleaseEvent(QMouseEvent *e)
 		     (e->pos().x() <= m_mouse_down_x+dmin)) )
 	    {
 		// deselect if only clicked without moving
-		unsigned int x = m_offset + pixels2samples(e->pos().x());
-		selectRange(x, 0);
+		unsigned int pos = m_offset + pixels2samples(e->pos().x());
+		selectRange(pos, 0);
 		setMouseMode(MouseNormal);
+		showPosition(0, 0, 0, QPoint(-1,-1));
 	    }
 	    break;
 	}
@@ -917,6 +1052,229 @@ void SignalWidget::mouseReleaseEvent(QMouseEvent *e)
 
 }
 
+//***************************************************************************
+SignalWidget::PositionWidget::PositionWidget(QWidget *parent)
+    :QWidget(parent), m_label(0), m_alignment(0),
+     m_radius(10), m_arrow_length(30)
+{
+    m_label = new QLabel(this);
+    Q_ASSERT(m_label);
+    if (!m_label) return;
+
+    m_label->setFrameStyle(QFrame::Panel | QFrame::Plain);
+    m_label->setPalette(QToolTip::palette()); // use same colors as a QToolTip
+    m_label->setFocusPolicy(QWidget::NoFocus);
+    m_label->setMouseTracking(true);
+    m_label->setLineWidth(0);
+
+    setPalette(QToolTip::palette()); // use same colors as a QToolTip
+    setFocusPolicy(QWidget::NoFocus);
+    setMouseTracking(true);
+
+    hide();
+}
+
+//***************************************************************************
+SignalWidget::PositionWidget::~PositionWidget()
+{
+    if (m_label) delete m_label;
+    m_label = 0;
+}
+
+//***************************************************************************
+void SignalWidget::PositionWidget::setText(const QString &text, int alignment)
+{
+    if (!m_label) return;
+
+    m_alignment = alignment;
+
+    m_label->setText(text);
+    m_label->setAlignment(m_alignment);
+    m_label->resize(m_label->sizeHint());
+
+    switch (m_alignment) {
+	case AlignLeft:
+	    resize(m_arrow_length + m_radius + m_label->width() + m_radius,
+	           m_radius + m_label->height() + m_radius);
+	    m_label->move(m_arrow_length + m_radius, m_radius);
+	    break;
+	case AlignRight:
+	    resize(m_radius + m_label->width() + m_radius + m_arrow_length,
+	           m_radius + m_label->height() + m_radius);
+	    m_label->move(m_radius, m_radius);
+	    break;
+	case AlignHCenter:
+	    resize(m_radius + m_label->width() + m_radius,
+	           m_arrow_length + m_radius + m_label->height() + m_radius);
+	    m_label->move(m_radius, m_arrow_length + m_radius);
+	    break;
+	default:
+	    ;
+    }
+}
+
+//***************************************************************************
+bool SignalWidget::PositionWidget::event(QEvent *e)
+{
+    if (!e) return false;
+
+    // ignore any kind of event that might be of interest
+    // for the parent widget (in our case the SignalWidget)
+    switch (e->type()) {
+	case QEvent::MouseButtonPress:
+	case QEvent::MouseButtonRelease:
+	case QEvent::MouseButtonDblClick:
+	case QEvent::MouseMove:
+	case QEvent::KeyPress:
+	case QEvent::KeyRelease:
+	case QEvent::Accel:
+	case QEvent::Wheel:
+	case QEvent::Clipboard:
+	case QEvent::Speech:
+	case QEvent::DragEnter:
+	case QEvent::DragMove:
+	case QEvent::DragLeave:
+	case QEvent::Drop:
+	case QEvent::DragResponse:
+	case QEvent::TabletMove:
+	case QEvent::TabletPress:
+	case QEvent::TabletRelease:
+	    return false;
+	default:
+	    ;
+    }
+
+    // everything else: let it be handled by QLabel
+    return QWidget::event(e);
+}
+
+//***************************************************************************
+void SignalWidget::PositionWidget::paintEvent(QPaintEvent *)
+{
+    QPainter p;
+
+    QBitmap bmp(size());
+    bmp.fill(Qt::color0);
+    p.begin(&bmp);
+
+    QBrush brush(Qt::color1);
+    p.setBrush(brush);
+    p.setPen(Qt::color1);
+
+    const int h = height();
+    const int w = width();
+    QPointArray poly;
+
+    switch (m_alignment) {
+	case AlignLeft:
+	    poly.putPoints(0, 8,
+		m_arrow_length, 0,
+		w-1, 0,
+		w-1, h-1,
+		m_arrow_length, h-1,
+		m_arrow_length, 2*h/3,
+		0, h/2,
+		m_arrow_length, h/3,
+		m_arrow_length, 0
+	    );
+	    break;
+	case AlignRight:
+	    poly.putPoints(0, 8,
+		0, 0,
+		w-1-m_arrow_length, 0,
+		w-1-m_arrow_length, h/3,
+		w-1, h/2,
+		w-1-m_arrow_length, 2*h/3,
+		w-1-m_arrow_length, h-1,
+		0, h-1,
+		0, 0
+	    );
+	    break;
+	case AlignHCenter:
+	    break;
+	default:
+	    ;
+    }
+
+    p.drawPolygon(poly);
+    p.end();
+    setMask(bmp);
+
+    p.begin(this);
+    p.fillRect(rect(), colorGroup().background());
+    p.drawPolyline(poly);
+    p.end();
+}
+
+//***************************************************************************
+void SignalWidget::showPosition(const QString &text, unsigned int pos,
+                                double ms, const QPoint &mouse)
+{
+    int x = mouse.x();
+    int y = mouse.y();
+
+    // x/y == -1/-1 -> reset/hide the position
+    if ((x < 0) && (y < 0)) {
+	m_position_widget_timer.stop();
+	m_position_widget.hide();
+	return;
+    }
+
+    unsigned int t, h, m, s, tms;
+    t = (unsigned int)rint(ms * 10.0);
+    tms = t % 10000;
+    t /= 10000;
+    s = t % 60;
+    t /= 60;
+    m = t % 60;
+    t /= 60;
+    h = t;
+    QString hms;
+    hms.sprintf("%02u:%02u:%02u.%04u", h, m, s, tms);
+    QString txt = QString("%1\n%2\n%3").arg(text).arg(pos).arg(hms);
+
+    switch (selectionPosition(mouse.x()) & ~Selection) {
+	case LeftBorder:
+	    m_position_widget.setText(txt, AlignRight);
+	    x = samples2pixels(pos - m_offset) - m_position_widget.width();
+	    if (x < 0) {
+		// switch to left aligned mode
+		m_position_widget.setText(txt, AlignLeft);
+		x = samples2pixels(pos - m_offset);
+	    }
+	    break;
+	case RightBorder:
+	default:
+	    m_position_widget.setText(txt, AlignLeft);
+	    x = samples2pixels(pos - m_offset);
+	    if (x + m_position_widget.width() > width()) {
+		// switch to right aligned mode
+		m_position_widget.setText(txt, AlignRight);
+		x = samples2pixels(pos - m_offset) - m_position_widget.width();
+	    }
+	    break;
+    }
+
+    // adjust the position to avoid vertical clipping
+    int lh = m_position_widget.height();
+    if (y - lh/2 < 0) {
+	y = 0;
+    } else if (y + lh/2 > height()) {
+	y = height() - lh;
+    } else {
+	y -= lh/2;
+    }
+
+    m_position_widget.move(x, y);
+
+    if (!m_position_widget.isVisible())
+	m_position_widget.show();
+    m_position_widget.repaint(false);
+
+    m_position_widget_timer.stop();
+    m_position_widget_timer.start(POSITION_WIDGET_TIME, true);
+}
+
 //***************************************************************************
 void SignalWidget::mouseMoveEvent(QMouseEvent *e)
 {
@@ -930,42 +1288,99 @@ void SignalWidget::mouseMoveEvent(QMouseEvent *e)
     // abort if no signal is loaded
     if (!m_signal_manager.length()) return;
 
+    // there seems to be a BUG in Qt, sometimes e->pos() produces flicker/wrong
+    // coordinates on the start of a fast move!?
+    // globalPos() seems not to have this effect
+    int mouse_x = mapFromGlobal(e->globalPos()).x();
+    int mouse_y = mapFromGlobal(e->globalPos()).y();
+    if (mouse_x < 0) mouse_x = 0;
+    if (mouse_y < 0) mouse_y = 0;
+    if (mouse_x >= width())  mouse_x = width()  - 1;
+    if (mouse_y >= height()) mouse_y = height() - 1;
+    QPoint pos(mouse_x, mouse_y);
+
+    // bail out if the position did not change
+    static int last_x = -1;
+    static int last_y = -1;
+    if ((mouse_x == last_x) && (mouse_y == last_y))
+	return;
+    last_x = mouse_x;
+    last_y = mouse_y;
+
     switch (m_mouse_mode) {
 	case MouseSelect: {
 	    // in move mode, a new selection was created or an old one grabbed
 	    // this does the changes with every mouse move...
-	    int mx = e->pos().x();
-	    if (mx < 0) mx = 0;
-	    if (mx >= m_width) mx = m_width-1;
-
-	    unsigned int x = m_offset + pixels2samples(mx);
+	    unsigned int x = m_offset + pixels2samples(mouse_x);
 	    m_selection->update(x);
-
-	    unsigned int len = m_selection->right() - m_selection->left() + 1;
-	    selectRange(m_selection->left(), len);
+	    selectRange(m_selection->left(), m_selection->length());
+	    showPosition(i18n("selection"), x, samples2ms(x), pos);
 	    break;
 	}
 	default: {
+	    unsigned int first = m_signal_manager.selection().first();
+	    unsigned int last  = m_signal_manager.selection().last();
+	    Label *label = findLabelNearMouse(mouse_x);
+
+	    // find out what is nearer: label or selection border ?
+	    if (label && (first != last) && isSelectionBorder(mouse_x)) {
+		const unsigned int pos = m_offset + pixels2samples(mouse_x);
+		const unsigned int d_label = (pos > label->pos()) ?
+		    (pos - label->pos()) : (label->pos() - pos);
+		const unsigned int d_left = (pos > first) ?
+		    (pos - first) : (first - pos);
+		const unsigned int d_right = (pos > last) ?
+		    (pos - last) : (last - pos);
+		if ( ((d_label ^ 2) > (d_left ^ 2)) &&
+		     ((d_label ^ 2) > (d_right ^ 2)) )
+		{
+		    // selection borders are nearer
+		    label = 0;
+		}
+	    }
+
 	    // yes, this code gives the nifty cursor change....
-	    int x = e->pos().x();
-	    if (isSelectionBorder(x)) {
+	    if (label) {
 		setMouseMode(MouseAtSelectionBorder);
-	    } else if (isInSelection(x)) {
+		QString text = i18n("label #%1").arg(labelIndex(label));
+		if (label->name().length())
+		    text += i18n(" (%1)").arg(label->name());
+		showPosition(text, label->pos(), samples2ms(label->pos()),
+		    pos);
+		break;
+	    } else if ((first != last) && isSelectionBorder(mouse_x)) {
+		setMouseMode(MouseAtSelectionBorder);
+		switch (selectionPosition(mouse_x) & ~Selection) {
+		    case LeftBorder:
+			showPosition(i18n("selection, left border"),
+			    first, samples2ms(first), pos);
+			break;
+		    case RightBorder:
+			showPosition(i18n("selection, right border"),
+			    last, samples2ms(last), pos);
+			break;
+		    default:
+			showPosition(0, 0, 0, QPoint(-1,-1));
+		}
+	    } else if (isInSelection(mouse_x)) {
 		setMouseMode(MouseInSelection);
+		showPosition(0, 0, 0, QPoint(-1,-1));
                 int dmin = KGlobalSettings::dndEventDelay();
 		if ((e->state() & Qt::LeftButton) &&
-		    ((e->pos().x() < m_mouse_down_x-dmin) ||
-		     (e->pos().x() > m_mouse_down_x+dmin)) )
+		    ((mouse_x < m_mouse_down_x - dmin) ||
+		     (mouse_x > m_mouse_down_x + dmin)) )
 		{
 		    startDragging();
 		}
 	    } else {
 		setMouseMode(MouseNormal);
+		showPosition(0, 0, 0, QPoint(-1,-1));
 	    }
 	}
     }
 }
 
+//***************************************************************************
 void SignalWidget::paintEvent(QPaintEvent *)
 {
     InhibitRepaintGuard inhibit(*this, false); // avoid recursion
@@ -1054,12 +1469,27 @@ void SignalWidget::paintEvent(QPaintEvent *)
 	Q_ASSERT(m_layer[LAYER_MARKERS]);
 	if (!m_layer[LAYER_MARKERS]) return;
 
-//	qDebug("SignalWidget::paintEvent(): - redraw of markers layer -");
+// 	qDebug("SignalWidget::paintEvent(): - redraw of markers layer -");
 	m_layer[LAYER_MARKERS]->fill(black);
 
-// 	p.begin(m_layer[LAYER_MARKERS]);
-	// ### nothing to do yet
-// 	p.end();
+	QPainter p;
+	p.begin(m_layer[LAYER_MARKERS]);
+
+	QPtrListIterator<Label> it(m_labels);
+	Label *label;
+	while ((label = it.current())) {
+	    ++it;
+	    unsigned int pos = label->pos();
+	    if (pos < m_offset) continue; // outside left
+	    int x = samples2pixels(pos - m_offset);
+	    if (x >= m_width) continue; // outside right
+
+	    p.setPen(Qt::cyan);
+	    p.setRasterOp(XorROP);
+	    p.drawLine(x, 0, x, m_height);
+	}
+
+	p.end();
 
 	m_update_layer[LAYER_MARKERS] = false;
 	update_pixmap = true;
@@ -1091,8 +1521,8 @@ void SignalWidget::paintEvent(QPaintEvent *)
 		if (right >= (unsigned int)(m_width)) right=m_width-1;
 		if (left > right) left = right;
 
+		p.setPen(yellow);
 		if (left == right) {
-		    p.setPen (green);
 		    p.drawLine(left, 0, left, m_height);
 		} else {
 		    p.setBrush(yellow);
@@ -1179,14 +1609,14 @@ double SignalWidget::samples2ms(unsigned int samples)
 }
 
 //***************************************************************************
-unsigned int SignalWidget::pixels2samples(int pixels)
+unsigned int SignalWidget::pixels2samples(int pixels) const
 {
     if ((pixels < 0) || (m_zoom <= 0.0)) return 0;
     return (unsigned int)rint((double)pixels * m_zoom);
 }
 
 //***************************************************************************
-int SignalWidget::samples2pixels(int samples)
+int SignalWidget::samples2pixels(int samples) const
 {
     if (m_zoom==0.0) return 0;
     return (int)rint((double)samples / m_zoom);
@@ -1199,14 +1629,70 @@ int SignalWidget::displaySamples()
 }
 
 //***************************************************************************
-void selectMarkers(const char */*command*/)
+Label *SignalWidget::findLabelNearMouse(int x) const
 {
-//    Parser parser(command);
+    const int tol = SELECTION_TOLERANCE;
+    unsigned int pos = m_offset + pixels2samples(x);
+    Label *nearest = 0;
+    int dmin = pixels2samples(SELECTION_TOLERANCE) ^ 2;
+
+    QPtrListIterator<Label> it(m_labels);
+    Label *label;
+    while ((label = it.current())) {
+	++it;
+	unsigned int lp = label->pos();
+	if (lp < m_offset) continue; // outside left
+	int lx = samples2pixels(lp - m_offset);
+	if (lx >= m_width) continue; // outside right
+
+	if ((lx + tol < x) || (lx > x + tol))
+	    continue; // out of tolerance
+
+	int dist = (pos - label->pos()) ^ 2;
+	if (dist < dmin) {
+	    // found a new "nearest" label
+	    dmin = dist;
+	    nearest = label;
+	}
+    }
+
+    return nearest;
 }
 
 //***************************************************************************
-LabelType *findMarkerType (const char */*txt*/)
+Label *SignalWidget::findLabel(unsigned int pos) const
 {
+    QPtrListIterator<Label> it(m_labels);
+    Label *label;
+    while ((label = it.current())) {
+	if (label->pos() == pos) return label; // found it
+	++it;
+    }
+    return 0; // nothing found
+}
+
+//***************************************************************************
+int SignalWidget::labelIndex(const Label *label) const
+{
+    int index = 0;
+    QPtrListIterator<Label> it(m_labels);
+    while (const Label *l = it.current()) {
+	if (l == label) return index; // found it
+	index++;
+ 	++it;
+    }
+    return -1; // nothing found*/
+}
+
+//***************************************************************************
+Label *SignalWidget::labelAtIndex(int index)
+{
+    return m_labels.at(index);
+}
+
+//***************************************************************************
+// LabelType *findMarkerType (const char */*txt*/)
+// {
 //    int cnt = 0;
 //    LabelType *act;
 //
@@ -1215,28 +1701,9 @@ LabelType *findMarkerType (const char */*txt*/)
 //	cnt++;
 //    }
 //    qWarning("could not find Labeltype %s\n", txt);
-    return 0;
-}
+//     return 0;
+// }
 
-////***************************************************************************
-//void SignalWidget::deleteLabel ()
-//{
-//    if (!signalmanage) return ;
-//
-//    Label *tmp;
-//    int l = signalmanage->getLMarker();
-//    int r = signalmanage->getRMarker();
-//
-//    for (tmp = labels->first(); tmp; tmp = labels->next()) {
-//	int pos = ms2samples(tmp->pos);
-//	if ( (pos >= l) && (pos < r) ) {
-//	    labels->remove(tmp);
-//	    tmp = labels->first();
-//	}
-//    }
-//    refreshLayer(LAYER_MARKERS);
-//}
-//
 ////****************************************************************************
 //void SignalWidget::loadLabel()
 //{
@@ -1297,52 +1764,135 @@ LabelType *findMarkerType (const char */*txt*/)
 //	fclose (out);
 //    }
 //}
-//
-////****************************************************************************
-//void SignalWidget::addLabel (const char *params)
-//{
-//    if (signalmanage && markertype) {
-//	Parser parser(params);
-//	Label *newmark;
-//
-//	if (parser.countParams() > 0) {
-//	    newmark = new Label (params);
-//	} else {
-//	    double pos = ((double)signalmanage->getLMarker()) * 1000 / signalmanage->getRate();
-//	    newmark = new Label (pos, markertype);
-//
-//	    //should it need a name ?
-//	    if (markertype->named) {
-//		Dialog *dialog =
-//		    DynamicLoader::getDialog ("stringenter", new DialogOperation("Enter name of label :", true));    //create a modal dialog
-//
-//		if (dialog) {
-//		    dialog->show ();
-//
-//		    if (dialog->result()) {
-//			printf ("dialog:%s\n", dialog->getCommand());
-//			newmark->setName (dialog->getCommand());
-//			delete dialog;
-//		    } else {
-//			delete newmark;
-//			newmark = 0;
-//		    }
-//		} else {
-//		    KMessageBox::message (this, "Error", i18n("Dialog not loaded !"));
-//		    delete newmark;
-//		    newmark = 0;
-//		}
-//	    }
-//	}
-//
-//	if (newmark) {
-//	    labels->inSort (newmark);
-//
-//	    refreshLayer(LAYER_MARKERS);
-//	}
-//    }
-//}
-//
+
+//***************************************************************************
+bool SignalWidget::labelProperties(Label *label)
+{
+    if (!label) return false;
+
+    LabelPropertiesWidget *dlg =
+	new LabelPropertiesWidget(this, "Label Properties");
+    Q_ASSERT(dlg);
+    if (!dlg) return false;
+
+    dlg->setLabelIndex(labelIndex(label));
+    dlg->setLabelPosition(label->pos(), m_signal_manager.length(),
+	m_signal_manager.rate());
+    dlg->setLabelName(label->name());
+
+    bool accepted = (dlg->exec() == QDialog::Accepted);
+    if (accepted) {
+	UndoTransactionGuard undo(m_signal_manager, i18n("modify label"));
+	UndoModifyLabelAction *undo_modify =
+	    new UndoModifyLabelAction(*this, *label);
+	if (!m_signal_manager.registerUndoAction(undo_modify)) {
+	    delete undo_modify;
+	    delete dlg;
+	    return false;
+	}
+
+	// move the label and change the name if possible
+	label->moveTo(dlg->labelPosition());
+	label->rename(dlg->labelName());
+	dlg->saveSettings();
+
+	// now store the label's current position,
+	// for finding it again later
+	undo_modify->setLastPosition(label->pos());
+
+	// NOTE: moving might also change the index, so the complete
+	//       markers layer has to be refreshed
+	refreshLayer(LAYER_MARKERS);
+    }
+
+    delete dlg;
+    return accepted;
+}
+
+//***************************************************************************
+void SignalWidget::addLabel(unsigned int pos)
+{
+    // if there already is a label at the given position, do nothing
+    if (findLabel(pos)) return;
+
+    // create a new label
+    Label *label = new Label(pos, "");
+    Q_ASSERT(label);
+    if (!label) {
+	KMessageBox::sorry(this, i18n("Out of memory"));
+	return;
+    }
+
+    // put the label into the list
+    m_labels.inSort(label);
+
+    // register the undo action
+    UndoTransactionGuard undo(m_signal_manager, i18n("add label"));
+    UndoAddLabelAction *undo_add =
+	new UndoAddLabelAction(*this, labelIndex(label));
+    if (!m_signal_manager.registerUndoAction(undo_add)) {
+	delete undo_add;
+	m_labels.remove(label);
+	return;
+    }
+
+    // edit the label's properties
+    labelProperties(label);
+
+    refreshLayer(LAYER_MARKERS);
+    showPosition(0, 0, 0, QPoint(-1,-1));
+}
+
+//***************************************************************************
+Label *SignalWidget::addLabel(unsigned int pos, const QString &name)
+{
+    // if there already is a label at the given position, do nothing
+    if (findLabel(pos)) return 0;
+
+    // create a new label
+    Label *label = new Label(pos, name);
+    Q_ASSERT(label);
+    if (!label) {
+	KMessageBox::sorry(this, i18n("Out of memory"));
+	return 0;
+    }
+
+    // put the label into the list
+    m_labels.inSort(label);
+    refreshLayer(LAYER_MARKERS);
+    showPosition(0, 0, 0, QPoint(-1,-1));
+
+    return label;
+}
+
+//***************************************************************************
+void SignalWidget::deleteLabel(int index, bool with_undo)
+{
+    Q_ASSERT(index >= 0);
+    Q_ASSERT(index < (int)m_labels.count());
+    if ((index < 0) || (index >= (int)m_labels.count())) return;
+
+    Label *label = m_labels.at(index);
+    Q_ASSERT(label);
+    if (!label) return;
+
+    // register the undo action
+    if (with_undo) {
+	UndoTransactionGuard undo(m_signal_manager, i18n("delete label"));
+	UndoDeleteLabelAction *undo_del =
+	    new UndoDeleteLabelAction(*this, *label);
+	if (!m_signal_manager.registerUndoAction(undo_del)) {
+	    delete undo_del;
+	    delete label;
+	    return;
+	}
+    }
+
+    m_labels.remove(label);
+    refreshLayer(LAYER_MARKERS);
+    showPosition(0, 0, 0, QPoint(-1,-1));
+}
+
 ////****************************************************************************
 //void SignalWidget::jumptoLabel ()
 // another fine function contributed by Gerhard Zintel
@@ -1697,20 +2247,6 @@ LabelType *findMarkerType (const char */*txt*/)
 //	}
 //    return i;
 //}
-//
-////*****************************************************************************
-//void SignalWidget::addLabelType (LabelType *marker)
-//{
-//    globals.markertypes.append (marker);
-//    if (manage) manage->addNumberedMenuEntry ("ID_LABELS_TYPE", (char *)marker->name);
-//}
-//
-////*****************************************************************************
-//void SignalWidget::addLabelType (const char *str)
-//{
-//    LabelType *marker = new LabelType(str);
-//    if (marker) addLabelType (marker);
-//}
 
 //***************************************************************************
 void SignalWidget::playbackStopped()
@@ -1749,7 +2285,7 @@ void SignalWidget::slotTrackInserted(unsigned int index, Track &track)
             this, SLOT(refreshSignalLayer()));
 
     connect(&track, SIGNAL(sigSelectionChanged()),
-	    this, SIGNAL(sigTrackSeclecionChanged()));
+	    this, SIGNAL(sigTrackSelectionChanged()));
     connect(&track, SIGNAL(sigSelectionChanged()),
             this, SLOT(refreshSignalLayer()));
 }
@@ -1769,19 +2305,59 @@ void SignalWidget::slotTrackDeleted(unsigned int index)
 }
 
 //***************************************************************************
-void SignalWidget::slotSamplesInserted(unsigned int /*track*/,
-    unsigned int /*offset*/, unsigned int /*length*/)
+void SignalWidget::slotSamplesInserted(unsigned int track,
+    unsigned int offset, unsigned int length)
 {
-//    qDebug("SignalWidget(): slotSamplesInserted(%u, %u,%u)", track,
-//	offset, length);
+//     qDebug("SignalWidget(): slotSamplesInserted(%u, %u,%u)", track,
+// 	offset, length);
+
+    // only adjust the labels once per operation
+    if (track != m_signal_manager.selectedTracks().at(0)) return;
+
+    unsigned int modified = 0;
+    QPtrListIterator<Label> it(m_labels);
+    while (Label *label = it.current()) {
+	unsigned int pos = label->pos();
+	if (pos >= offset) {
+	    label->moveTo(pos + length);
+	    modified++;
+	}
+	++it;
+    }
+
+    if (modified)
+	refreshLayer(LAYER_MARKERS);
 }
 
 //***************************************************************************
-void SignalWidget::slotSamplesDeleted(unsigned int /*track*/,
-    unsigned int /*offset*/, unsigned int /*length*/)
+void SignalWidget::slotSamplesDeleted(unsigned int track,
+    unsigned int offset, unsigned int length)
 {
-//    qDebug("SignalManager(): slotSamplesDeleted(%u, %u,%u)", track,
-//	offset, length);
+//    qDebug("SignalWidget(): slotSamplesDeleted(%u, %u...%u)", track,
+// 	offset, offset+length-1);
+
+    // only adjust the labels once per operation
+    if (track != m_signal_manager.selectedTracks().at(0)) return;
+
+    unsigned int modified = 0;
+    QPtrListIterator<Label> it(m_labels);
+    while (Label *label = it.current()) {
+	unsigned int pos = label->pos();
+	if (pos >= offset + length) {
+	    // move label left
+	    label->moveTo(pos - length);
+	    modified++;
+	} else if ((pos >= offset) && (pos < offset+length)) {
+	    // delete the label
+	    deleteLabel(labelIndex(label), true);
+	    modified++;
+	    continue;
+	}
+	++it;
+    }
+
+    if (modified)
+	refreshLayer(LAYER_MARKERS);
 }
 
 //***************************************************************************
