@@ -20,11 +20,14 @@
 
 #include "config.h"
 #include <qdragobject.h>
+#include <qlabel.h>
 #include <qptrlist.h>
 #include <qtimer.h>
 #include <qpainter.h>
 #include <qwidget.h>
 
+#include "libkwave/LabelList.h"
+
 #include "PlaybackController.h"
 #include "SignalManager.h"
 
@@ -155,6 +158,41 @@ public:
     /** Returns the playback controller */
     PlaybackController &playbackController();
 
+    /**
+     * add a new label
+     * @param pos position of the label [samples]
+     */
+    void addLabel(unsigned int pos);
+
+    /**
+     * add a new label, without undo
+     * @param pos position of the label [samples]
+     * @param name the name of the label
+     * @return pointer to the new created label
+     */
+    Label *addLabel(unsigned int pos, const QString &name);
+
+    /**
+     * delete an existing label
+     * @param index the index of the label [0...N-1]
+     * @param with_undo if true, create undo info
+     */
+    void deleteLabel(int index, bool with_undo);
+
+    /**
+     * find a label by it's index
+     * @param index the index of the label [0...N-1]
+     * @return pointer to the label or null pointer if not found
+     */
+    Label *labelAtIndex(int index);
+
+    /**
+     * Returns the index of a label, counting from zero
+     * @param label pointer to a Label
+     * @return index [0...N-1] or -1 if label is a null pointer
+     */
+    int labelIndex(const Label *label) const;
+
 public slots:
 
     /**
@@ -246,6 +284,20 @@ protected:
     /** @see Qt XDND documentation */
     virtual void dragMoveEvent(QDragMoveEvent *event);
 
+protected:
+
+    friend class UndoModifyLabelAction;
+
+    /**
+     * returns the label at a given exact position
+     * @param pos position of the label [samples]
+     * @return the label at the position or null if not found
+     */
+    Label *findLabel(unsigned int pos) const;
+
+    /** Refreshes the layer with the markers */
+    void refreshMarkersLayer();
+
 protected slots:
 
     /**
@@ -257,6 +309,9 @@ protected slots:
      */
     void allowRepaint(bool repaint);
 
+    /** Handler for context menus */
+    void contextMenuEvent(QContextMenuEvent *e);
+
 private slots:
 
     /**
@@ -339,6 +394,33 @@ private slots:
      */
     void timedRepaint();
 
+    /** context menu: "edit/undo" */
+    void contextMenuEditUndo()   { forwardCommand("undo()"); };
+
+    /** context menu: "edit/redo" */
+    void contextMenuEditRedo()   { forwardCommand("redo()"); };
+
+    /** context menu: "edit/cut" */
+    void contextMenuEditCut()    { forwardCommand("cut()"); };
+
+    /** context menu: "edit/copy" */
+    void contextMenuEditCopy()   { forwardCommand("copy()"); };
+
+    /** context menu: "edit/paste" */
+    void contextMenuEditPaste()  { forwardCommand("paste()"); };
+
+    /** context menu: "save selection" */
+    void contextMenuSaveSelection()  { forwardCommand("saveselect()"); };
+
+    /** context menu: "label / new" */
+    void contextMenuLabelNew();
+
+    /** context menu: "label / delete" */
+    void contextMenuLabelDelete();
+
+    /** context menu: "label / properties..." */
+    void contextMenuLabelProperties();
+
 signals:
 
     /**
@@ -393,7 +475,7 @@ signals:
     void sigTrackDeleted(unsigned int track);
 
     /** The selection state of at least one track has changed */
-    void sigTrackSeclecionChanged();
+    void sigTrackSelectionChanged();
 
 protected:
 
@@ -441,6 +523,45 @@ protected:
 	bool m_repaint;
     };
 
+    class PositionWidget: public QWidget
+    {
+    public:
+	/** Constructor */
+	PositionWidget(QWidget *parent);
+
+	/** Destructor */
+	virtual ~PositionWidget();
+
+	/**
+	 * set a new label text and alignment
+	 * @param text the text of the label, can be multiline and rtf/html
+	 * @param alignment the alignment of the label and the widget,
+	 *                  can be AlignLeft, AlignRight or AlignHCenter
+	 */
+	virtual void setText(const QString &text, int alignment);
+
+    protected:
+	/** event filter */
+	virtual bool event(QEvent *e);
+
+	/** paint event: draws the text and the arrow */
+	virtual void paintEvent(QPaintEvent *);
+
+    private:
+
+	/** the label that contains the text */
+	QLabel *m_label;
+
+	/** alignment of the label / text */
+	int m_alignment;
+
+	/** the radius of the corners [pixel] */
+	int m_radius;
+
+	/** the length of the arrows [pixel] */
+	int m_arrow_length;
+    };
+
     /**
      * Returns the zoom value that will be used to fit the whole signal
      * into the current window.
@@ -463,16 +584,30 @@ protected:
     /** slot for repainting the widget or portions of it */
     void paintEvent(QPaintEvent *);
 
+    /**
+     * Returns the label that is nearest to the given mouse position
+     * and is visible or null if none found.
+     * @param x mouse position, X coordinate, relative to the widget
+     * @return nearest label or null if none found.
+     */
+    Label *findLabelNearMouse(int x) const;
+
+    /**
+     * Opens a dialog for editing the properties of a label
+     * @param label a Label that should be edited
+     * @return true if the dialog has been accepted,
+     *         otherwise false (canceled)
+     */
+    bool labelProperties(Label *label);
+
 //    void loadLabel ();
 //    void appendLabel ();
 //    void deleteLabel ();
 //    void saveLabel (const char *);
-//    void addLabel (const char *);
 //    void jumptoLabel ();
 //    void markSignal (const char *);
 //    void markPeriods (const char *);
 //    void savePeriods ();
-//    bool executeLabelCommand(const QString &command);
 
     /**
      * Handles commands for navigation and selection.
@@ -502,7 +637,7 @@ private:
      * @param pixels pixel offset
      * @return index of the sample
      */
-    unsigned int pixels2samples(int pixels);
+    unsigned int pixels2samples(int pixels) const;
 
     /**
      * Converts a pixel offset into a sample index using the current zoom
@@ -510,7 +645,7 @@ private:
      * @param samples number of samples to be converted
      * @return pixel offset
      */
-    int samples2pixels(int samples);
+    int samples2pixels(int samples) const;
 
     /**
      * Fixes the zoom and the offset of the display so that no non-existing
@@ -525,6 +660,17 @@ private:
      */
     void setMouseMode(MouseMode mode);
 
+    /**
+     * Shows the current cursor position as a tooltip
+     * @param text description of the position
+     * @param pos marker position [samples]
+     * @param ms marker position in milliseconds
+     * @param mouse the coordinates of the mouse cursor,
+     *              relative to this widget [pixel]
+     */
+    void showPosition(const QString &text, unsigned int pos, double ms,
+                      const QPoint &mouse);
+
 private:
 
     /** Pixmaps for buffering each layer */
@@ -586,12 +732,12 @@ private:
     /** list of track pixmaps */
     QPtrList<TrackPixmap> m_track_pixmaps;
 
+    /** list of labels */
+    LabelList m_labels;
+
     /** pixmap for avoiding flicker */
     QPixmap *m_pixmap;
 
-//    LabelList *labels;           //linked list of markers
-//    LabelType *markertype;       //selected marker type
-
     /** mode of the mouse cursor */
     MouseMode m_mouse_mode;
 
@@ -604,6 +750,12 @@ private:
     /** timer for limiting the number of repaints per second */
     QTimer m_repaint_timer;
 
+    /** small widget for showing the mouse cursor position */
+    PositionWidget m_position_widget;
+
+    /** timer for automatic hiding */
+    QTimer m_position_widget_timer;
+
 };
 
 #endif /* _SIGNAL_WIDGET_H_ */
