@@ -847,7 +847,7 @@ int TopWidget::openRecent(const QString &str)
 int TopWidget::openFile()
 {
     KwaveFileDialog dlg(":<kwave_open_dir>", CodecManager::decodingFilter(),
-        this, "Kwave open file", true, 0, "*.wav");
+        this, "Kwave open file", true, 0, 0);
     dlg.setMode(static_cast<KFile::Mode>(KFile::File | KFile::ExistingOnly));
     dlg.setOperationMode(KFileDialog::Opening);
     dlg.setCaption(i18n("Open"));
@@ -947,7 +947,7 @@ int TopWidget::saveFileAs(bool selection)
 	    // mimetype
 	    Q_ASSERT(m_plugin_manager);
 	    res = (m_plugin_manager) ?
-	        res = m_plugin_manager->setupPlugin("fileinfo") : -1;
+	        m_plugin_manager->setupPlugin("fileinfo") : -1;
 
 	    // restore the mime type and the filename
 	    signalManager().fileInfo().set(INF_MIMETYPE,
