@@ -39,6 +39,13 @@ class UndoAction
 
 public:
 
+    /** IDs for gouping undo actions */
+    enum UndoGroupID {
+	Default = 0,         /**< default, no special order */
+	LabelEdit     =  99, /**< label edit: before label command */
+	LabelCommands = 100  /**< label commands: have to be undone later */
+    };
+
     /** Destructor */
     virtual ~UndoAction() {};
 
@@ -81,6 +88,12 @@ public:
      */
     virtual UndoAction *undo(SignalManager &manager, bool with_redo) = 0;
 
+    /**
+     * Returns the group in which the undo action has to be sorted in
+     * @return the numeric id of the group
+     */
+    virtual UndoGroupID group() const { return Default; };
+
 };
 
 #endif /* _UNDO_ACTION_H_ */
