@@ -1,3 +1,20 @@
+/***************************************************************************
+          MenuManager.h  -  manager class for Kwave's menu structure
+			     -------------------
+    begin                : Sun Jun 4 2000
+    copyright            : (C) 2000 by Thomas Eschenbacher
+    email                : Thomas.Eschenbacher@gmx.de
+ ***************************************************************************/
+
+/***************************************************************************
+ *                                                                         *
+ *   This program is free software; you can redistribute it and/or modify  *
+ *   it under the terms of the GNU General Public License as published by  *
+ *   the Free Software Foundation; either version 2 of the License, or     *
+ *   (at your option) any later version.                                   *
+ *                                                                         *
+ ***************************************************************************/
+
 #ifndef _MENU_MANAGER_H_
 #define _MENU_MANAGER_H_
 
@@ -32,10 +49,8 @@ public:
      */
     MenuManager(QWidget *parent, KMenuBar &bar);
 
-    /**
-     * Destructor.
-     */
-    ~MenuManager();
+    /** Destructor. */
+    virtual ~MenuManager();
 
     /**
      * Executes menu commands.
