@@ -1,5 +1,5 @@
 /***************************************************************************
-			  MenuSub.cpp  -  description
+			  MenuSub.cpp  -  submenu
 			     -------------------
     begin                : Mon Jan 10 2000
     copyright            : (C) 2000 by Thomas Eschenbacher
