@@ -19,16 +19,21 @@
 #define _CURVE_H_
 
 #include "config.h"
-#include <qarray.h>
+#include <qmemarray.h>
 #include <qobject.h>
 #include <qstring.h>
 
 #include "Interpolation.h"
-#include "PointSet.h"
 
 class Curve: public QObject
 {
 public:
+    /** specialized point class with double precision */
+    class Point {
+    public:
+        double x;
+        double y;
+    };
 
     /**
      * Default constructor, creates an empty curve.
@@ -131,7 +136,7 @@ public:
     /**
      * Returns a point after a given point or null if the
      * given point was the first one.
-     * @param act the point before the one we look for
+     * @param p the point before the one we look for
      */
     Point *next(Point *p);
 
@@ -170,7 +175,7 @@ public:
      * @return Array of interpolated values or null if the
      *         number of points was zero or the curve was empty.
      */
-    QArray<double> interpolation(unsigned int points);
+    QMemArray<double> interpolation(unsigned int points);
 
 protected:
     /**
@@ -184,7 +189,7 @@ protected:
 private:
 
     /** list of points, sorted by x coordinates */
-    QList<Point> m_points;
+    QPtrList<Point> m_points;
 
     /** interpolation object */
     Interpolation m_interpolation;
