@@ -295,16 +295,6 @@ unsigned int Stripe::resize(unsigned int length, bool initialize)
 	}
     }
 
-    if (length < old_length) {
-	// something has been deleted from the end
-	unsigned int change = old_length - length;
-	emit sigSamplesDeleted(*this, length, change);
-    } else if (length > old_length) {
-	// something has been added to the end
-	unsigned int change = length - old_length;
-	emit sigSamplesInserted(*this, old_length, change);
-    }
-
     return length;
 }
 
@@ -339,59 +329,9 @@ unsigned int Stripe::append(const QMemArray<sample_t> &samples,
 	    / sizeof(sample_t);
     }
 //     qDebug("Stripe::append(): resized to %d", m_length);
-
-    // something has been added to the end
-    if (appended) emit sigSamplesInserted(*this, old_length, appended);
-
     return appended;
 }
 
-//***************************************************************************
-unsigned int Stripe::insert(const QMemArray<sample_t> &samples,
-                            unsigned int offset, unsigned int count)
-{
-    unsigned int old_length;
-    unsigned int inserted = 0;
-
-    {
-	QMutexLocker lock(&m_lock_samples);
-
-	if (!count || !m_length) return 0; // nothing to do
-	Q_ASSERT(offset + count <= samples.size());
-	if (offset + count > samples.size()) return 0;
-
-	qDebug("Stripe::insert: inserting %d samples", count);
-
-	old_length = m_length;
-	unsigned int new_length = old_length + count;
-	if (resizeStorage(new_length) != new_length) {
-	    qWarning("Stripe::insert(): m_length=%u, old=%u, wanted=%u",
-		m_length, old_length, new_length);
-	    return 0;
-	}
-
-	if (offset < old_length) {
-	    // move old samples right
-	    MappedArray _samples(*this, m_length);
-
-	    unsigned int src = offset;
-	    unsigned int dst = offset + count;
-	    if (!_samples.copy(dst, src, old_length - offset)) return 0;
-	}
-
-	// insert the new samples at the given offset
-	MappedArray _samples(*this, m_length);
-	inserted = _samples.copy(offset, samples, 0, count);
-    }
-
-    qDebug("Stripe::insert(): resized to %d", m_length);
-
-    // something has been inserted
-    if (inserted) emit sigSamplesInserted(*this, offset, inserted);
-
-    return inserted;
-}
-
 //***************************************************************************
 void Stripe::deleteRange(unsigned int offset, unsigned int length)
 {
@@ -433,8 +373,6 @@ void Stripe::deleteRange(unsigned int offset, unsigned int length)
 	// resize the buffer to it's new size
 	resizeStorage(size - length);
     }
-
-    // sigSamplesDeleted will be emitted in the Track, not here
 }
 
 //***************************************************************************
@@ -442,16 +380,11 @@ void Stripe::overwrite(unsigned int offset,
 	const QMemArray<sample_t> &source,
 	unsigned int srcoff, unsigned int srclen)
 {
-    unsigned int count = srclen;
-    {
-	QMutexLocker lock(&m_lock_samples);
+    QMutexLocker lock(&m_lock_samples);
 
-	count = MemoryManager::instance().writeTo(m_storage,
-	    offset * sizeof(sample_t),
-	    &source[srcoff], count * sizeof(sample_t)) / sizeof(sample_t);
-    }
-
-    if (count) emit sigSamplesModified(m_start + offset, count);
+    MemoryManager::instance().writeTo(m_storage,
+	offset * sizeof(sample_t),
+	&source[srcoff], srclen * sizeof(sample_t));
 }
 
 //***************************************************************************
