@@ -89,6 +89,7 @@ Stripe *Track::appendStripe(unsigned int length)
 
 	s = newStripe(start, len);
 	if (!s) break;
+	if (len) emit sigSamplesInserted(*this, start, len);
 
 	length -= len;
 	start  += len;
@@ -98,31 +99,15 @@ Stripe *Track::appendStripe(unsigned int length)
     return s;
 }
 
-//***************************************************************************
-void Track::connectStripe(Stripe *s)
-{
-    Q_ASSERT(s);
-    if (!s) return;
-
-    connect(s, SIGNAL(sigSamplesDeleted(Stripe&, unsigned int, unsigned int)),
-	this, SLOT(slotSamplesDeleted(Stripe&, unsigned int, unsigned int)));
-    connect(s, SIGNAL(sigSamplesInserted(Stripe&, unsigned int, unsigned int)),
-	this, SLOT( slotSamplesInserted(Stripe&, unsigned int, unsigned int)));
-    connect(s, SIGNAL(sigSamplesModified(unsigned int, unsigned int)),
-	this, SLOT(slotSamplesModified(unsigned int, unsigned int)));
-}
-
 //***************************************************************************
 Stripe *Track::newStripe(unsigned int start, unsigned int length)
 {
     Stripe *s = new Stripe(start);
     Q_ASSERT(s);
-//  qDebug("Track::newStripe(%u, %u): new stripe at %p", start, length, s);
     if (!s) return 0;
+//  qDebug("Track::newStripe(%u, %u): new stripe at %p", start, length, s);
 
-    connectStripe(s);
-
-    if (length) s->resize(length);
+    s->resize(length);
 
     return s;
 }
@@ -132,19 +117,6 @@ void Track::deleteStripe(Stripe *s)
 {
     if (!s) return;
 
-    disconnect(s, SIGNAL(sigSamplesDeleted(Stripe&, unsigned int,
-	    unsigned int)),
-	    this, SLOT(slotSamplesDeleted(Stripe&, unsigned int,
-	    unsigned int)));
-    disconnect(s, SIGNAL(sigSamplesInserted(Stripe&, unsigned int,
-	    unsigned int)),
-	    this, SLOT( slotSamplesInserted(Stripe&, unsigned int,
-	    unsigned int)));
-    disconnect(s, SIGNAL(sigSamplesModified(unsigned int,
-	                                    unsigned int)),
-	    this, SLOT(slotSamplesModified(unsigned int,
-	                                   unsigned int)));
-
     m_stripes.setAutoDelete(true);
     m_stripes.remove(s);
 }
@@ -168,8 +140,6 @@ void Track::splitStripe(Stripe *stripe, unsigned int offset)
     qDebug("Track::splitStripe(%p, %u): new stripe at [%u ... %u] (%u)",
            stripe, offset, s->start(), s->end(), s->length());
 
-    connectStripe(s);
-
     qDebug("    inserting at list index %u", m_stripes.findRef(stripe)+1);
     m_stripes.insert(m_stripes.findRef(stripe)+1, s);
 }
@@ -358,27 +328,6 @@ void Track::select(bool selected)
     emit sigSelectionChanged();
 }
 
-//***************************************************************************
-void Track::slotSamplesInserted(Stripe &src, unsigned int offset,
-                                unsigned int length)
-{
-    emit sigSamplesInserted(*this, src.start()+offset, length);
-}
-
-//***************************************************************************
-void Track::slotSamplesDeleted(Stripe &src, unsigned int offset,
-                               unsigned int length)
-{
-    emit sigSamplesDeleted(*this, src.start()+offset, length);
-}
-
-//***************************************************************************
-void Track::slotSamplesModified(unsigned int offset,
-                                unsigned int length)
-{
-    emit sigSamplesModified(*this, offset, length);
-}
-
 //***************************************************************************
 void Track::appendAfter(Stripe *stripe,  unsigned int offset,
                         const QMemArray<sample_t> &buffer,
@@ -396,8 +345,8 @@ void Track::appendAfter(Stripe *stripe,  unsigned int offset,
 	if (len + stripe->length() > STRIPE_LENGTH_MAXIMUM)
 	    len = STRIPE_LENGTH_MAXIMUM - stripe->length();
 
-// 	qDebug("Track::appendAfter(): appending %u samples to %p",
-// 	       len, stripe);
+	qDebug("Track::appendAfter(): appending %u samples to %p",
+	       len, stripe);
 	stripe->append(buffer, buf_offset, len);
 
 	offset     += len;
@@ -421,6 +370,7 @@ void Track::appendAfter(Stripe *stripe,  unsigned int offset,
 
 	// append to the new stripe
 	new_stripe->append(buffer, buf_offset, len);
+
 	qDebug("new stripe: [%u ... %u]", new_stripe->start(),
 	       new_stripe->end());
 
@@ -468,6 +418,7 @@ void Track::writeSamples(InsertMode mode,
 	case Append: {
 // 	    qDebug("writeSamples() - Append");
 	    appendAfter(m_stripes.last(), offset, buffer, buf_offset, length);
+	    emit sigSamplesInserted(*this, offset, length);
 	    break;
 	}
 	case Insert: {
@@ -511,29 +462,18 @@ void Track::writeSamples(InsertMode mode,
 		moveRight(offset, length);
 		appendAfter(stripe_before, offset, buffer,
 		            buf_offset, length);
-		break;
-	    }
-
-	    if (!target_stripe) {
-		// insert somewhere before, between or after stripes
-		moveRight(offset, length);
-		appendAfter(stripe_before, offset, buffer,
-		            buf_offset, length);
+		emit sigSamplesInserted(*this, offset, length);
 		break;
 	    }
 
 	    // if no stripe was found, create a new one and
 	    // insert it between the existing ones
 	    if (!target_stripe) {
-		target_stripe = newStripe(offset, 0);
-		Q_ASSERT(target_stripe);
-		if (!target_stripe) return;
-
-		// insert into our stripes, if the stripe before
-		// is null, the new one will be prepended
+		// insert somewhere before, between or after stripes
 		moveRight(offset, length);
-		int index = m_stripes.findRef(stripe_before) + 1;
-		m_stripes.insert(index, target_stripe);
+		appendAfter(stripe_before, offset, buffer,
+		            buf_offset, length);
+		emit sigSamplesInserted(*this, offset, length);
 	    } else {
 	        // split the target stripe and insert the samples
 		// between the two new ones
@@ -541,6 +481,7 @@ void Track::writeSamples(InsertMode mode,
 		moveRight(offset, length);
 		appendAfter(target_stripe, offset, buffer,
 		            buf_offset, length);
+		emit sigSamplesInserted(*this, offset, length);
 	    }
 
 	    break;
@@ -554,6 +495,7 @@ void Track::writeSamples(InsertMode mode,
 		// -> append mode
 		qDebug("- no stripes -> appending at zero");
 		appendAfter(0, offset, buffer, buf_offset, length);
+		emit sigSamplesInserted(*this, offset, length);
 		break;
 	    }
 
@@ -594,6 +536,7 @@ void Track::writeSamples(InsertMode mode,
 //		qDebug("L: [%u ....... %u]",start, end);
 //		qDebug("    [%u ... %u]", left, left+len-1);
 		s->overwrite(left-start, buffer, buf_offset, len);
+		emit sigSamplesModified(*this, left, len);
 //		qDebug("L: overwrite done.");
 
 		buf_offset += len;
@@ -636,6 +579,7 @@ void Track::writeSamples(InsertMode mode,
 // 		qDebug("R: [%u ....... %u]",start, end);
 // 		qDebug("R:     [%u ... %u]", end-len+1, end);
 		s->overwrite(0, buffer, buf_offset + (start-left), len); // ###
+		emit sigSamplesModified(*this,start, len);
 // 		qDebug("R: overwrite done.");
 
 		buf_offset += len;
@@ -679,6 +623,10 @@ void Track::writeSamples(InsertMode mode,
 		stripe_before = 0;
 
 	    appendAfter(stripe_before, left, buffer, buf_offset, length);
+	    if (stripe_before == m_stripes.last())
+		emit sigSamplesInserted(*this, offset, length);
+	    else
+		emit sigSamplesModified(*this, offset, length);
 
 	    // look for possible fragmentation at the left side
 
