@@ -20,21 +20,39 @@
 
 #include <qstring.h>
 
+/**
+ * enum for the known playback methods
+ */
+typedef enum {
+    PLAYBACK_NONE = 0, /**< none selected */
+    PLAYBACK_ARTS,     /**< aRts sound daemon */
+    PLAYBACK_OSS,      /**< OSS native or ALSA OSS emulation */
+    PLAYBACK_ALSA,     /**< ALSA native */
+    PLAYBACK_JACK,     /**< Jack sound daemon */
+    PLAYBACK_INVALID   /**< (keep this the last entry, EOL delimiter) */
+} playback_method_t;
+
+/** post-increment operator for the playback method */
+inline playback_method_t &operator ++(playback_method_t &m) {
+    return (m = (m < PLAYBACK_INVALID) ?
+                (playback_method_t)((int)(m) + 1) : m);
+}
+
 /**
  * A class that contains all necessary parameters for
  * setting up (initializing) a playback device.
  */
 class PlayBackParam
 {
-public:    
+public:
     /** Default constructor */
     PlayBackParam()
         :rate(44100), channels(2), bits_per_sample(16),
-        device("/dev/dsp"), bufbase(10)
+        device("/dev/dsp"), bufbase(10),
+        method(PLAYBACK_ARTS)
     {
-	
     };
-    
+
     /** Sample rate [samples/second] */
     double rate;
 
@@ -49,6 +67,10 @@ public:
 
     /** base of the buffer size (buffer size will be 2^bufbase) */
     unsigned int bufbase;
+
+    /** method/class to use for playback */
+    playback_method_t method;
+
 };
 
 #endif /* _PLAY_BACK_PARAM_H_ */
