@@ -19,9 +19,10 @@
 #define _PLAY_BACK_PLUGIN_H_
 
 #include "config.h"
+
+#include <qmutex.h>
 #include <qstring.h>
 
-#include "mt/Mutex.h"
 #include "mt/SignalProxy.h"
 #include "libkwave/KwavePlugin.h"
 #include "libkwave/PlaybackDeviceFactory.h"
@@ -31,6 +32,7 @@ class QStringList;
 class PlaybackController;
 class PluginContext;
 class PlayBackDevice;
+class PlayBackDialog;
 
 class PlayBackPlugin: public KwavePlugin, public PlaybackDeviceFactory
 {
@@ -77,7 +79,13 @@ public slots:
      */
     void stopDevicePlayBack();
 
+    /**
+     * Plays a sample sound for testing the playback
+     */
+    void testPlayBack();
+
 private slots:
+
     /**
      * Called from the playback thread to notify about a new
      * playback pointer.
@@ -85,6 +93,18 @@ private slots:
      */
     void updatePlaybackPos();
 
+    /**
+     * Change the playback method
+     * @param method the new playback method
+     */
+    void setMethod(playback_method_t method);
+
+    /**
+     * Change the playback device (file) name
+     * @param device the filename of the device
+     */
+    void setDevice(const QString &device);
+
 protected:
 
     /**
@@ -139,11 +159,14 @@ protected:
 
 private:
 
+    /** dialog for the playback setup */
+    PlayBackDialog *m_dialog;
+
     /** The playback device used for playback */
     PlayBackDevice *m_device;
 
     /** Mutex for locking acces to the playback device */
-    Mutex m_lock_device;
+    QMutex m_lock_device;
 
     /** the parameters used for playback */
     PlayBackParam m_playback_params;
