@@ -27,6 +27,7 @@
 #include "RecordParams.h"
 #include "RecordState.h"
 #include "RecordTypesMap.h"
+#include "StatusWidget.h"
 
 #include "libkwave/Sample.h"
 #include "libkwave/SampleFormat.h"
@@ -128,6 +129,12 @@ public:
      */
     void updateEffects(unsigned int track, QMemArray<sample_t> &buffer);
 
+    /**
+     * Show the "source" device tab, usually if the setup was
+     * not successful.
+     */
+    void showDevicePage();
+
 signals:
 
     /** emits changes in the currently selected record method */
@@ -174,6 +181,9 @@ public slots:
     /** updates the number of recorded samples */
     void setRecordedSamples(unsigned int samples_recorded);
 
+    /** show a message in the status bar */
+    void message(const QString &message);
+
 private slots:
 
     /**
@@ -298,6 +308,9 @@ private:
     /** if false, do nothing in setDevice */
     bool m_enable_setDevice;
 
+    /** widget with a icon in the status bar */
+    StatusWidget *m_state_icon_widget;
+
 };
 
 #endif /* _RECORD_PLUGIN_H_ */
