@@ -148,6 +148,9 @@ private slots:
     /** restart recorder with new buffer settings */
     void buffersChanged();
 
+    /** the prerecording checkbos has changed */
+    void prerecordingChanged(bool enable);
+
 private:
 
     /** close m_device and delete it */
@@ -203,6 +206,12 @@ private:
      */
     void flushPrerecordingQueue();
 
+    /**
+     * Returns true if all parameters are valid and the recording
+     * (thread) could be started.
+     */
+    bool paramsValid();
+
 private:
 
     /** last recording method */
